/* ccgo: igs/result.cc
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "../config.hh"
#include "../gettext.h"
#include "result.hh"
#include "player.hh"
#include <sstream>
#include <regexx.hh>
#include <iostream>
#define atms(i, j) (rx.match[i].atom[j].str())

#include <cstdio>
#define _(String) Glib::locale_to_utf8(gettext(String))

using namespace igs;

Result::Result()
{
	ok = false;
}

bool Result::build(const std::string & s)
{
	raw = s;
	regexx::Regexx rx;
	if (! rx.exec(s, "(\\S+)\\s+\\[(.{4})\\]\\((B|W)\\) : (\\S+)\\s+\\[(.{4})\\]\\S+ H (.) K\\s+(\\S+) (\\d+)x\\d+ \\S\\S(.+)")) {
		ok = false;
		std::cerr << s << std::endl;
		return false;
	}
	if (atms(0, 2) == "B") {
		winer = go::TURN_BLACK;
		white = atms(0, 3);
		w_rank = Player::Rank(atms(0, 4));
		black = atms(0, 0);
		b_rank = Player::Rank(atms(0, 1));
	} else {
		winer = go::TURN_WHITE;
		white = atms(0, 0);
		w_rank = Player::Rank(atms(0, 1));
		black = atms(0, 3);
		b_rank = Player::Rank(atms(0, 4));
	}
	std::istringstream(atms(0, 5)) >> handicap;
	std::istringstream(atms(0, 6)) >> komi;
	std::istringstream(atms(0, 7)) >> board_size;
	std::string sec = atms(0, 8);
	if (! rx.exec(sec, "(\\S+) (\\d+)-(\\d+)-(\\d+) (\\S)")) {
		ok = false;
		return false;
	}
	if (atms(0, 0) == "Time") win_type = WIN_TIME;
	else if (atms(0, 0) == "Resign") win_type = WIN_RESIGN;
	else {
		win_type = WIN_COUNT;
		std::istringstream(atms(0, 0)) >> win;
	}
	std::istringstream(atms(0, 1)) >> date;
	std::istringstream(atms(0, 2)) >> hour;
	std::istringstream(atms(0, 3)) >> minute;

	rated = atms(0, 4) == "R";
	ok = true;
	return true;
}

Result::operator bool () const
{
	return ok;
}

const Glib::ustring Result::short_text() const
{
	Glib::ustring u;
	if (winer == go::TURN_BLACK) {
		u = black + "[" + b_rank.get_text() + "]" + _("(B)") + "-" + white + "[" + w_rank.get_text() + "]";
	} else {
		u = white + "[" + w_rank.get_text() + "]" + _("(W)") + "-" + black + "[" + b_rank.get_text() + "]";
	}

	char buf[100];
	switch (win_type) {
	case WIN_COUNT:
		snprintf(buf, 100, _("+%g").c_str(), win);
		u.append(buf);
		break;
	case WIN_RESIGN:
		u += _("+resign");
		break;
	case WIN_TIME:
		u += _("+time");
		break;
	}
	if (rated) u += "*";
	return u;
}

const Glib::ustring Result::short_text(const std::string & n) const
{
	Glib::ustring u;

	Glib::ustring w;
	Glib::ustring b;
	if (white == n) {
		w = n + _("(W)");
		b = black + "[" + b_rank.get_text() + "]";
	} else if (black == n) {
		b = n + _("(B)");
		w = white + "[" + w_rank.get_text() + "]";
	} else {
		b = black + "[" + b_rank.get_text() + "]" + _("(B)");
		w = white + "[" + w_rank.get_text() + "]" + _("(W)");
	}
	if (winer == go::TURN_BLACK) {
		u = b + "-" + w;
	} else {
		u = w + "-" + b;
	}

	char buf[100];
	switch (win_type) {
	case WIN_COUNT:
		snprintf(buf, 100, _("+%g").c_str(), win);
		u.append(buf);
		break;
	case WIN_RESIGN:
		u += _("+resign");
		break;
	case WIN_TIME:
		u += _("+time");
		break;
	}
	if (rated) u += "*";
	return u;
}
